package cn.wzpmc.entities;

import cn.wzpmc.DreamRunnerMod;
import cn.wzpmc.DreamRunnerModClient;
import cn.wzpmc.sounds.MusicSoundInstance;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.network.ClientPlayerEntity;
import net.minecraft.client.sound.SoundInstance;
import net.minecraft.client.sound.SoundManager;
import net.minecraft.client.texture.NativeImage;
import net.minecraft.client.texture.NativeImageBackedTexture;
import net.minecraft.util.Identifier;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.*;

@Environment(EnvType.CLIENT)
public class MusicObject {
    private final String name;
    private final long songId;
    private final int duringTime;
    private final List<String> artists;
    private final Map<Long, String> lrc = new HashMap<>();
    private final List<Long> lrcTime = new ArrayList<>();
    private final SoundInstance soundInstance;
    private byte[] picByteArray;
    private NativeImage image;
    private NativeImageBackedTexture backedTexture;
    private Identifier imageId;
    private long startT;

    public MusicObject(String name, long songId, List<String> artists, int duringTime, byte[] picByteArray, String lrc, byte[] songByteArray) {
        this.name = name;
        this.songId = songId;
        this.artists = artists;
        this.duringTime = duringTime;
        this.picByteArray = picByteArray;
        this.startT = new Date().getTime();
        String[] lineSplit = lrc.split("\n");
        for (String fileLine : lineSplit) {
            String[] timeLrcSplit = fileLine.split("]");
            String time = timeLrcSplit[0].replace("[", "");
            String[] msSplit = time.split("\\.");
            String[] minSecSplit = msSplit[0].split(":");
            long min = Long.parseLong(minSecSplit[0]);
            long sec = Long.parseLong(minSecSplit[1]);
            long ms = Long.parseLong(msSplit[1]);
            StringBuilder line = new StringBuilder();
            for (int i = 1; i < timeLrcSplit.length; i++) {
                line.append(timeLrcSplit[i]);
            }
            long lrcTime = (min * 60 + sec) * 1000 + ms;
            this.lrc.put(lrcTime, line.toString());
            this.lrcTime.add(lrcTime);
        }
        this.lrcTime.sort(Comparator.comparingLong(Long::longValue).reversed());
        // soundInstance
        ClientPlayerEntity player = MinecraftClient.getInstance().player;
        soundInstance = new MusicSoundInstance(player, new ByteArrayInputStream(songByteArray), this);
    }

    public static String getTimeString(long duringTime) {
        long secDuringTime = duringTime / 1000;
        long sec = secDuringTime % 60;
        long min = secDuringTime / 60;
        return String.format("%d:%02d", min, sec);
    }

    public int getDuringTime() {
        return duringTime;
    }

    public List<String> getArtists() {
        return artists;
    }

    public String getDuring() {
        return getTimeString(getDuringLong());
    }

    public long getDuringLong() {
        long l = new Date().getTime() - startT;
        if (l >= this.duringTime) {
            DreamRunnerModClient.musics.remove(this);
            if (!DreamRunnerModClient.musics.isEmpty()) {
                MusicObject next = DreamRunnerModClient.musics.get(0);
                next.play();
            }
            return this.duringTime;
        }
        return l;
    }

    public void start() {
        this.startT = new Date().getTime();
    }

    public void play() {
        SoundManager soundManager = MinecraftClient.getInstance().getSoundManager();
        soundManager.play(soundInstance);
    }

    public NativeImage getImage() {
        if (image == null) {
            try {
                BufferedImage originalImage = ImageIO.read(new ByteArrayInputStream(this.picByteArray));
                BufferedImage resizedImage = new BufferedImage(64, 64, BufferedImage.TYPE_INT_RGB);
                Graphics2D g2d = resizedImage.createGraphics();
                g2d.drawImage(originalImage, 0, 0, 64, 64, null);
                g2d.dispose();
                ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                ImageIO.write(resizedImage, "png", byteArrayOutputStream);
                byte[] byteArray = byteArrayOutputStream.toByteArray();
                this.picByteArray = byteArray;
                this.image = NativeImage.read(new ByteArrayInputStream(byteArray));
            } catch (IOException e) {
                DreamRunnerMod.LOGGER.error("读取图片失败！", e);
                return null;
            }
        }
        return image;
    }

    public NativeImageBackedTexture getBackedTexture() {
        if (backedTexture == null) {
            backedTexture = new NativeImageBackedTexture(this.getImage());
        }
        return backedTexture;
    }

    public Identifier getImageId() {
        if (this.imageId == null) {
            this.imageId = MinecraftClient.getInstance().getTextureManager().registerDynamicTexture("der", this.getBackedTexture());
        }
        return this.imageId;
    }

    public String getLrc() {
        long dt = this.getDuringLong();
        for (Long time : this.lrcTime) {
            if (time <= dt) {
                return this.lrc.get(time);
            }
        }
        return "";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MusicObject that = (MusicObject) o;
        return songId == that.songId && duringTime == that.duringTime && Arrays.equals(picByteArray, that.picByteArray) && Objects.equals(artists, that.artists);
    }

    @Override
    public int hashCode() {
        int result = Objects.hash(songId, duringTime, artists);
        result = 31 * result + Arrays.hashCode(picByteArray);
        return result;
    }

    public String getName() {
        return name;
    }
}
