package cn.wzpmc.sounds;

import cn.wzpmc.DreamRunnerMod;
import cn.wzpmc.entities.MusicObject;
import javazoom.jl.decoder.*;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.sound.AudioStream;

import javax.sound.sampled.AudioFormat;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Environment(EnvType.CLIENT)
public class MyAudioStream implements AudioStream {
    private final Bitstream bitstream;
    private final Decoder decoder;
    private final AudioFormat format;
    private final MusicObject object;
    private boolean isStart;

    public MyAudioStream(InputStream inputStream, MusicObject object) throws BitstreamException, IOException {
        this.bitstream = new Bitstream(inputStream);
        this.decoder = new Decoder();
        this.object = object;
        this.isStart = false;
        Header header = bitstream.readFrame();
        if (header == null) {
            throw new IOException("读取MP3文件头失败！");
        }
        this.format = new AudioFormat(header.frequency(), 16, header.mode() == Header.SINGLE_CHANNEL ? 1 : 2, true, false);
        bitstream.unreadFrame();
    }

    @Override
    public void close() {
        try {
            this.bitstream.close();
        } catch (Exception e) {
            DreamRunnerMod.LOGGER.error("关闭资源失败！", e);
        }
    }

    @Override
    public AudioFormat getFormat() {
        return this.format;
    }

    @Override
    public ByteBuffer getBuffer(int size) {
        if (!this.isStart) {
            this.object.start();
            isStart = true;
        }
        ByteBuffer buffer = ByteBuffer.allocateDirect(size).order(ByteOrder.nativeOrder());
        try {
            while (buffer.hasRemaining()) {
                Header frameHeader = bitstream.readFrame();
                if (frameHeader == null) {
                    break;
                }

                SampleBuffer output = (SampleBuffer) decoder.decodeFrame(frameHeader, bitstream);

                short[] pcm = output.getBuffer();
                for (int i = 0; i < output.getBufferLength(); i++) {
                    if (!buffer.hasRemaining()) {
                        break;
                    }
                    buffer.putShort(pcm[i]);
                }

                bitstream.closeFrame();
            }
        } catch (Exception e) {
            DreamRunnerMod.LOGGER.error("读取音频文件失败！", e);
            return null;
        }
        buffer.flip();
        return buffer;
    }
}
